#//-----------------------------------------------
#//TUSB-S01VM用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_07_23　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#Device DLL Loader ( cdecl )
tusbs01vmdll = cdll.LoadLibrary("TUSBSVM.DLL")

#######  dll function definitions (do not call directry from user codes)
tusbs01vmdll.Tusbs01vm_Device_Open.restype = c_short
tusbs01vmdll.Tusbs01vm_Device_Open.argtypes = (c_short,)
tusbs01vmdll.Tusbs01vm_Device_Close.restype = None
tusbs01vmdll.Tusbs01vm_Device_Close.argtypes = (c_short,)
tusbs01vmdll.Tusbs01vm_Single_Sample.restype = c_short
tusbs01vmdll.Tusbs01vm_Single_Sample.argtypes = (c_short,POINTER(c_double))
tusbs01vmdll.Tusbs01vm_Start_Sample.restype = c_short
tusbs01vmdll.Tusbs01vm_Start_Sample.argtypes = (c_short,c_short)
tusbs01vmdll.Tusbs01vm_Get_Datas.restype = c_short
tusbs01vmdll.Tusbs01vm_Get_Datas.argtypes = (c_short,POINTER(c_double),POINTER(c_ubyte),POINTER(c_ubyte))
tusbs01vmdll.Tusbs01vm_Get_DevType.restype = c_short
tusbs01vmdll.Tusbs01vm_Get_DevType.argtypes = (c_short,POINTER(c_ubyte))

#######  support functions (do not call directry from user codes)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):
        return False
    elif(Val < Min):
        return False
    elif(Val > Max):
        return False
    else:
        return True

def IsValidId(DevId):
    return IsInRange(DevId,0,3)

######## def functions   ###############################################

def Tusbs01vm_Device_Open(DevId):
    """Open Device
    
    Arguments:
    DevId           ----    Device Id 0~3
    
    Returns:
    int             ----    Result code
    """
    if( IsValidId(DevId)):
        return tusbs01vmdll.Tusbs01vm_Device_Open(DevId)
    else:
        return 1

def Tusbs01vm_Device_Close(DevId):
    """Close Device
    
    Arguments:
    DevId           ----    Device Id 0~3
    """
    if( IsValidId(DevId)):
        tusbs01vmdll.Tusbs01vm_Device_Close(DevId)

def Tusbs01vm_Single_Sample(DevId):
    """Get Data
    
    Arguments:
    DevId           ----    Device Id 0~3

    Returns:
    int,            ----    Result code
    float,          ----    Measured data
    """
    if not IsValidId(DevId): return 1,None
    Data = c_double()
    ret = tusbs01vmdll.Tusbs01vm_Single_Sample(DevId,byref(Data))
    if ret != 0:
        return ret,None
    return 0,Data.value

def Tusbs01vm_Start_Sample(DevId,Rate):
    """Start Sampling
    
    Arguments:
    DevId           ----    Device Id 0~3
    Rate            ----    Sampling Rate 1~10000
    
    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    if not IsInRange(Rate,1,10000): return 8
    return tusbs01vmdll.Tusbs01vm_Start_Sample(DevId,Rate)

def Tusbs01vm_Get_Datas(DevId,ReqLen):
    """Get Sampled Data
    Arguments:
    DevId           ----    Device Id 0~3
    ReqLen          ----    Request Data Length 1 ~ 128

    Returns:
    int,            ----    Reuslt code
    double list,    ----    Sampled data list
    int list,       ----    Data Number list
    """
    if not IsValidId(DevId): return 1,None,None,None
    if not IsInRange(ReqLen,1,128): return 8,None,None,None
    Size = c_ubyte(ReqLen)
    Data = ( c_double * ReqLen )()
    SerNum = ( c_byte * ReqLen )()
    p_Data = cast(pointer(Data),POINTER(c_double))
    p_SerNum = cast(pointer(SerNum),POINTER(c_ubyte))
    ret = tusbs01vmdll.Tusbs01vm_Get_Datas(DevId,p_Data,p_SerNum,byref(Size))
    if ret != 0: return ret,None,None,None
    rData = [Data[i] for i in range(Size.value)]
    rSerNum = [SerNum[i] for i in range(Size.value)]
    return 0,rData,rSerNum

def Tusbs01vm_Get_DevType(DevId):
    """Get Device Type
    
    Arguments:
    DevId           ----    Device Id 0~3

    Returns:
    int,            ----    Reuslt code
    int,            ----    Device type code
    """
    if not IsValidId(DevId): return 1,None
    Type = c_ubyte()
    ret = tusbs01vmdll.Tusbs01vm_Get_DevType(DevId,byref(Type))
    return ret,Type.value

#戻り値用エラーメッセージ
ErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            2:"ドライバがインストールされていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            7:"オープンされていません",
            8:"パラメータエラー",
            9:"USB通信エラーです"
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"

